# This file is part of the FTIO software 
#
# Copyright (c) 2023-2024, Technical University of Darmstadt, Germany
#
# This software may be modified and distributed under the terms of a BSD-style license.
# See the LICENSE file in the base directory for details.

__title__ = "FTIO"
__version__ = "0.0.4"
__license__ = "BSD 3"
__description__ = "FTIO: Capturing Periodic I/O Using Frequency Techniques"
__copyright__ = "Copyright (c) 2023-2024 Technical University of Darmstadt, Darmstadt, Germany"
__documentation_link__ = f"https://github.com/tuda-parallel/FTIO/tree/v{__version__}/docs"
__current_version_api__ = "https://pypi.org/pypi/extrap/json"
__developed_by_html__ = 'FTIO is developed by <a href="https://www.parallel.informatik.tu-darmstadt.de/">' 
__support_email__ = "ahmad.tarraf@tu-darmstadt.de"
__repo__='https://github.com/tuda-parallel/FTIO'
